package uulm.teamname.marvelous.gamelibrary.entities;

import uulm.teamname.marvelous.gamelibrary.IntVector2;

import java.util.ArrayList;

/** Represents an NPC inside the game. */
public class NPC extends Entity {
    /** The {@link Inventory} of the NPC */
    public final Inventory inventory;

    /**
     * Constructs a new {@link NPC}.
     * @param id The {@link EntityID} of the NPC
     * @param position The position of the NPC
     * @param inventory The starting inventory the NPC should have
     */
    public NPC(EntityID id, IntVector2 position, ArrayList<StoneType> inventory) {
        super(id, position);
        solid = false;
        opaque = true;
        this.inventory = new Inventory(inventory);
    }

    /**
     * Constructs a new {@link NPC} with an empty inventory.
     * @param id The {@link EntityID} of the NPC
     * @param position The position of the NPC
     */
    public NPC(EntityID id, IntVector2 position) {
        super(id, position);
        this.inventory = new Inventory();
    }

    @Override
    public NPC clone() {
        NPC clone = new NPC(id, position);
        for(StoneType stone: inventory) {
            clone.inventory.addStone(stone);
        }
        return clone;
    }
}
